@@ -1,12 +1,12 @@
-#  File R/InitWtErgmTerm.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File R/InitWtErgmTerm.R in package ergm, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2003-2020 Statnet Commons
-#######################################################################
+#  Copyright 2003-2021 Statnet Commons
+################################################################################
 
 binary_wrap <- function(InitFun, nw, a, valued_args, ddd, namemap = identity, cnmap = identity){
   namemap <- as_mapper(namemap)
@@ -46,7 +46,7 @@ InitWtErgmTerm.absdiff <- function(nw, arglist, ..., version=packageVersion("erg
   binary_dind_wrap("absdiff", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.absdiffcat <- function(nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.absdiffcat <- function(nw, arglist, ..., version=packageVersion("ergm")) {
   if(version <= as.package_version("3.9.4")){
     ### Check the network and arguments to make sure they are appropriate.
     a <- check.ErgmTerm(nw, arglist, directed=NULL, bipartite=NULL,
@@ -66,7 +66,7 @@ InitWtErgmTerm.absdiffcat <- function(nw, arglist, response, ..., version=packag
 }
 
 
-InitWtErgmTerm.atleast<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.atleast<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("threshold"),
                       vartypes = c("numeric"),
@@ -77,10 +77,10 @@ InitWtErgmTerm.atleast<-function(nw, arglist, response, ...) {
        inputs=a$threshold,
        dependence=FALSE,
        minval=0, maxval=network.dyadcount(nw,FALSE),
-       emptynwstats=if(0>=a$threshold) network.dyadcount(nw,FALSE) else 0)
+       emptynwstats=ifelse(0>=a$threshold, network.dyadcount(nw,FALSE), 0))
 }
 
-InitWtErgmTerm.atmost<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.atmost<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("threshold"),
                       vartypes = c("numeric"),
@@ -91,7 +91,7 @@ InitWtErgmTerm.atmost<-function(nw, arglist, response, ...) {
        inputs=a$threshold,
        dependence=FALSE,
        minval=0, maxval=network.dyadcount(nw,FALSE),
-       emptynwstats=if(0<=a$threshold) network.dyadcount(nw,FALSE) else 0)
+       emptynwstats=ifelse(0<=a$threshold, network.dyadcount(nw,FALSE), 0))
 }
 
 InitWtErgmTerm.b1cov<-function (nw, arglist, ..., version=packageVersion("ergm")) {
@@ -206,7 +206,7 @@ InitWtErgmTerm.diff <- function(nw, arglist, ..., version=packageVersion("ergm")
   binary_dind_wrap("diff", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.edgecov <- function(nw, arglist, response, ...) {
+InitWtErgmTerm.edgecov <- function(nw, arglist, ...) {
   ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, 
                       varnames = c("x", "attrname", "form"),
@@ -217,7 +217,7 @@ InitWtErgmTerm.edgecov <- function(nw, arglist, response, ...) {
 }
 
 
-InitWtErgmTerm.equalto<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.equalto<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("value", "tolerance"),
                       vartypes = c("numeric", "numeric"),
@@ -232,7 +232,7 @@ InitWtErgmTerm.equalto<-function(nw, arglist, response, ...) {
 }
 
 
-InitWtErgmTerm.ininterval<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.ininterval<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("lower","upper","open"),
                       vartypes = c("numeric","numeric","logical,character"),
@@ -261,7 +261,7 @@ InitWtErgmTerm.ininterval<-function(nw, arglist, response, ...) {
        ) network.dyadcount(nw,FALSE) else 0)
 }
 
-InitWtErgmTerm.greaterthan<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.greaterthan<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("threshold"),
                       vartypes = c("numeric"),
@@ -272,10 +272,10 @@ InitWtErgmTerm.greaterthan<-function(nw, arglist, response, ...) {
        inputs=a$threshold,
        dependence=FALSE,
        minval=0, maxval=network.dyadcount(nw,FALSE),
-       emptynwstats=if(0>a$threshold) network.dyadcount(nw,FALSE) else 0)
+       emptynwstats=ifelse(0>a$threshold, network.dyadcount(nw,FALSE), 0))
 }
 
-InitWtErgmTerm.smallerthan<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.smallerthan<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("threshold"),
                       vartypes = c("numeric"),
@@ -286,11 +286,11 @@ InitWtErgmTerm.smallerthan<-function(nw, arglist, response, ...) {
        inputs=a$threshold,
        dependence=FALSE,
        minval=0, maxval=network.dyadcount(nw,FALSE),
-       emptynwstats=if(0<a$threshold) network.dyadcount(nw,FALSE) else 0)
+       emptynwstats=ifelse(0<a$threshold, network.dyadcount(nw,FALSE), 0))
 }
 
 
-InitWtErgmTerm.sum<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.sum<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("pow"),
                       vartypes = c("numeric"),
@@ -309,78 +309,44 @@ InitWtErgmTerm.sum<-function(nw, arglist, response, ...) {
   }
 }
 
-InitWtErgmTerm.nodecovar<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.nodecovar<-function (nw, arglist, ...) {
   ### Check the network and arguments to make sure they are appropriate.
-  a <- check.ErgmTerm(nw, arglist, bipartite = FALSE,
-                      varnames = NULL,
-                      vartypes = NULL,
-                      defaultvalues = NULL,
-                      required = NULL)
+  a <- check.ErgmTerm(nw, arglist, directed = FALSE,
+                      varnames = c("center","transform"),
+                      vartypes = c("logical","character"),
+                      defaultvalues = list(FALSE,"identity"),
+                      required = c(FALSE,FALSE))
   ### Process the arguments
-
+  transcode <- match(match.arg(a$transform, c("identity","sqrt")), c("identity","sqrt"))-1  
   list(name="nodecovar",
        coef.names = "nodecovar",
+       inputs = c(transcode,a$center), # Transformation codes: 0 for no transformation, 1 for square root.
        dependence = TRUE
        )
 }
 
-InitWtErgmTerm.nodesqrtcovar<-function (nw, arglist, response, ...) {
-  ### Check the network and arguments to make sure they are appropriate.
-  a <- check.ErgmTerm(nw, arglist, bipartite = FALSE, nonnegative=TRUE, response=response,
-                      varnames = c("center"),
-                      vartypes = c("logical"),
-                      defaultvalues = list(TRUE),
-                      required = c(TRUE))
-  ### Process the arguments
-
-  name <- "nodesqrtcovar"
-
-  if(a$center) name <- paste(name,"centered",sep="_")
-
-  coef.name <- gsub("_",".",name)
-  
-  list(name=name,
-       coef.names = coef.name,
-       dependence = TRUE,
-       minval = if(a$center) NULL else 0
-       )
+InitWtErgmTerm.nodeosqrtcovar<-function (nw, arglist, ...) {
+  .Deprecated('nodeocovar(transform="sqrt")',
+            old = "nodeosqrtcovar")
+  arglist$transform <- "sqrt"
+  InitWtErgmTerm.nodeocovar(nw, arglist, ...)
 }
 
-InitWtErgmTerm.nodeosqrtcovar<-function (nw, arglist, response, ...) {
-  ### Check the network and arguments to make sure they are appropriate.
-  a <- check.ErgmTerm(nw, arglist, directed = TRUE, nonnegative=TRUE, response=response,
-                      varnames = c(),
-                      vartypes = c(),
-                      defaultvalues = list(),
-                      required = c())
-  ### Process the arguments
-
-  list(name="nodeosqrtcovar",
-       coef.names = "nodeosqrtcovar",
-       dependence = TRUE,
-       # arithmetic mean >= geometric mean
-       minval = 0
-       )
+InitWtErgmTerm.nodeisqrtcovar<-function (nw, arglist, ...) {
+  .Deprecated('nodeicovar(transform="sqrt")',
+            old = "nodeisqrtcovar")
+  arglist$transform <- "sqrt"
+  InitWtErgmTerm.nodeicovar(nw, arglist, ...)
 }
 
-InitWtErgmTerm.nodeisqrtcovar<-function (nw, arglist, response, ...) {
-  ### Check the network and arguments to make sure they are appropriate.
-  a <- check.ErgmTerm(nw, arglist, directed = TRUE, nonnegative=TRUE, response=response,
-                      varnames = c(),
-                      vartypes = c(),
-                      defaultvalues = list(FALSE),
-                      required = c(FALSE))
-  ### Process the arguments
-
-  list(name="nodeisqrtcovar",
-       coef.names = "nodeisqrtcovar",
-       dependence = TRUE,
-       # arithmetic mean >= geometric mean
-       minval = 0
-       )
+InitWtErgmTerm.nodesqrtcovar<-function (nw, arglist, ...) {
+  .Deprecated('nodecovar(transform="sqrt")',
+            old = "nodesqrtcovar")
+  arglist$transform <- "sqrt"
+  InitWtErgmTerm.nodecovar(nw, arglist, ...)
 }
 
-InitWtErgmTerm.nodefactor<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.nodefactor<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   ### Check the network and arguments to make sure they are appropriate.
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist,
@@ -400,7 +366,7 @@ InitWtErgmTerm.nodefactor<-function (nw, arglist, response, ..., version=package
 }
 
 
-InitWtErgmTerm.sociality<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.sociality<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   ### Check the network and arguments to make sure they are appropriate.
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=FALSE,
@@ -420,22 +386,23 @@ InitWtErgmTerm.sociality<-function (nw, arglist, response, ..., version=packageV
 }
 
 
-InitWtErgmTerm.nodeocovar<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.nodeocovar<-function (nw, arglist, ...) {
   ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, directed = TRUE,
-                      varnames = NULL,
-                      vartypes = NULL,
-                      defaultvalues = NULL,
-                      required = NULL)
+                      varnames = c("center","transform"),
+                      vartypes = c("logical","character"),
+                      defaultvalues = list(FALSE,"identity"),
+                      required = c(FALSE,FALSE))
   ### Process the arguments
-
+  transcode <- match(match.arg(a$transform, c("identity","sqrt")), c("identity","sqrt"))-1  
   list(name="nodeocovar",
        coef.names = "nodeocovar",
+       inputs = c(transcode,a$center), # Transformation codes: 0 for no transformation, 1 for square root.
        dependence = TRUE
        )
 }
 
-InitWtErgmTerm.nodeofactor<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.nodeofactor<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   ### Check the network and arguments to make sure they are appropriate.
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=TRUE, 
@@ -454,7 +421,7 @@ InitWtErgmTerm.nodeofactor<-function (nw, arglist, response, ..., version=packag
   binary_dind_wrap("nodeofactor", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.sender<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.sender<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   ### Check the network and arguments to make sure they are appropriate.
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=TRUE,
@@ -473,22 +440,23 @@ InitWtErgmTerm.sender<-function (nw, arglist, response, ..., version=packageVers
   binary_dind_wrap("sender", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.nodeicovar<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.nodeicovar<-function (nw, arglist, ...) {
   ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, directed = TRUE,
-                      varnames = NULL,
-                      vartypes = NULL,
-                      defaultvalues = NULL,
-                      required = NULL)
+                      varnames = c("center","transform"),
+                      vartypes = c("logical","character"),
+                      defaultvalues = list(FALSE,"identity"),
+                      required = c(FALSE,FALSE))
   ### Process the arguments
-
+  transcode <- match(match.arg(a$transform, c("identity","sqrt")), c("identity","sqrt"))-1  
   list(name="nodeicovar",
        coef.names = "nodeicovar",
+       inputs = c(transcode,a$center), # Transformation codes: 0 for no transformation, 1 for square root.
        dependence = TRUE
        )
 }
 
-InitWtErgmTerm.nodeifactor<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.nodeifactor<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   ### Check the network and arguments to make sure they are appropriate.
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=TRUE, 
@@ -507,7 +475,7 @@ InitWtErgmTerm.nodeifactor<-function (nw, arglist, response, ..., version=packag
   binary_dind_wrap("nodeifactor", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.receiver<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.receiver<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   ### Check the network and arguments to make sure they are appropriate.
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=TRUE,
@@ -566,7 +534,7 @@ InitWtErgmTerm.nodemix<-function (nw, arglist, ..., version=packageVersion("ergm
   binary_dind_wrap("nodemix", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.nodecov<-InitWtErgmTerm.nodemain<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.nodecov<-InitWtErgmTerm.nodemain<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist,
                         varnames = c("attrname","transform","transformname","form"),
@@ -583,7 +551,7 @@ InitWtErgmTerm.nodecov<-InitWtErgmTerm.nodemain<-function (nw, arglist, response
   binary_dind_wrap("nodecov", nw, a, ..., version=version)
 }
 
-InitWtErgmTerm.nodeicov<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.nodeicov<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=TRUE,
                         varnames = c("attrname","transform","transformname","form"),
@@ -602,7 +570,7 @@ InitWtErgmTerm.nodeicov<-function (nw, arglist, response, ..., version=packageVe
 }
 
 
-InitWtErgmTerm.nodeocov<-function (nw, arglist, response, ..., version=packageVersion("ergm")) {
+InitWtErgmTerm.nodeocov<-function (nw, arglist, ..., version=packageVersion("ergm")) {
   if(version <= as.package_version("3.9.4")){
     a <- check.ErgmTerm(nw, arglist, directed=TRUE, 
                         varnames = c("attrname","transform","transformname","form"),
@@ -621,7 +589,7 @@ InitWtErgmTerm.nodeocov<-function (nw, arglist, response, ..., version=packageVe
 }
 
 
-InitWtErgmTerm.edges<-InitWtErgmTerm.nonzero<-function(nw, arglist, response, ...) {
+InitWtErgmTerm.edges<-InitWtErgmTerm.nonzero<-function(nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = NULL,
                       vartypes = NULL,
@@ -634,7 +602,7 @@ InitWtErgmTerm.edges<-InitWtErgmTerm.nonzero<-function(nw, arglist, response, ..
        minval=0, maxval=network.dyadcount(nw,FALSE))
 }
 
-InitWtErgmTerm.mutual<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.mutual<-function (nw, arglist, ...) {
 ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, directed=TRUE, bipartite=NULL,
                       varnames = c("form","threshold"),
@@ -654,7 +622,7 @@ InitWtErgmTerm.mutual<-function (nw, arglist, response, ...) {
        )
 }
 
-InitWtErgmTerm.transitiveties<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.transitiveties<-function (nw, arglist, ...) {
 ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, bipartite=NULL,
                       varnames = c("threshold"),
@@ -669,13 +637,13 @@ InitWtErgmTerm.transitiveties<-function (nw, arglist, response, ...) {
        minval=0, maxval=network.dyadcount(nw,FALSE))  
 }
 
-InitWtErgmTerm.transitiveweights<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.transitiveweights<-function (nw, arglist, ...) {
 ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, bipartite=NULL, nonnegative=TRUE,
                       varnames = c("twopath","combine","affect"),
                       vartypes = c("character","character","character"),
                       defaultvalues = list("min","max","min"),
-                      required = c(FALSE,FALSE,FALSE), response=response)
+                      required = c(FALSE,FALSE,FALSE))
   twopaths<-c("min","geomean")
   twopath<-match.arg(a$twopath,twopaths)
   combines<-c("max", "sum")
@@ -694,7 +662,7 @@ InitWtErgmTerm.transitiveweights<-function (nw, arglist, response, ...) {
        minval = 0)
 }
 
-InitWtErgmTerm.cyclicalties<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.cyclicalties<-function (nw, arglist, ...) {
 ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, bipartite=NULL,
                       varnames = c("threshold"),
@@ -709,13 +677,13 @@ InitWtErgmTerm.cyclicalties<-function (nw, arglist, response, ...) {
   
 }
 
-InitWtErgmTerm.cyclicalweights<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.cyclicalweights<-function (nw, arglist, ...) {
 ### Check the network and arguments to make sure they are appropriate.
   a <- check.ErgmTerm(nw, arglist, bipartite=NULL, nonnegative=TRUE,
                       varnames = c("twopath","combine","affect"),
                       vartypes = c("character","character","character"),
                       defaultvalues = list("min","max","min"),
-                      required = c(FALSE,FALSE,FALSE), response=response)
+                      required = c(FALSE,FALSE,FALSE))
   twopaths<-c("min","geomean")
   twopath<-match.arg(a$twopath,twopaths)
   combines<-c("max", "sum")
@@ -734,7 +702,7 @@ InitWtErgmTerm.cyclicalweights<-function (nw, arglist, response, ...) {
        minval = 0)
 }
 
-InitWtErgmTerm.mm<-function (nw, arglist, response, ...) {
+InitWtErgmTerm.mm<-function (nw, arglist, ...) {
   a <- check.ErgmTerm(nw, arglist,
                       varnames = c("attrs", "levels", "levels2", "form"),
                       vartypes = c(ERGM_VATTR_SPEC, ERGM_LEVELS_SPEC, ERGM_LEVELS_SPEC, "character"),
