@@ -1,12 +1,12 @@
-#  File R/control.san.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File R/control.san.R in package ergm, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2003-2020 Statnet Commons
-#######################################################################
+#  Copyright 2003-2021 Statnet Commons
+################################################################################
 
 
 #' Auxiliary for Controlling SAN
@@ -30,8 +30,8 @@
 #' @param SAN.invcov Initial inverse covariance matrix used to
 #'   calculate Mahalanobis distance in determining how far a proposed
 #'   MCMC move is from the \code{target.stats} vector.  If `NULL`,
-#'   initially set to the identity matrix, then during subsequent runs
-#'   estimated empirically.
+#'   initially set to the identity matrix. In either case, during
+#'   subsequent runs, it is estimated empirically.
 #'
 #' @param SAN.invcov.diag Whether to only use the diagonal of the
 #'   covariance matrix. It seems to work better in practice.
@@ -42,8 +42,6 @@
 #'
 #' @param SAN.nsteps Number of MCMC proposals for all the annealing runs combined.
 #' @param SAN.samplesize Number of realisations' statistics to obtain for tuning purposes.
-#' @param SAN.init.maxedges Maximum number of edges expected in network.
-#' @param SAN.max.maxedges Hard upper bound on the number of edges in the network.
 #' @template control_MCMC_prop
 #' @param SAN.packagenames Names of packages in which to look for change
 #' statistic functions in addition to those autodetected. This argument should
@@ -63,9 +61,8 @@ control.san<-function(SAN.maxit=4,
                       SAN.nsteps.alloc=function(nsim) 2^seq_len(nsim),
                       SAN.nsteps=2^19,
                       SAN.samplesize=2^12,
-                      SAN.init.maxedges=20000,
-                      SAN.max.maxedges=2^26,
                       
+                      SAN.prop=trim_env(~sparse),
                       SAN.prop.weights="default",
                       SAN.prop.args=list(),
                       SAN.packagenames=c(),
@@ -77,10 +74,9 @@ control.san<-function(SAN.maxit=4,
                       seed=NULL,
                       parallel=0,
                       parallel.type=NULL,
-                      parallel.version.check=TRUE){
-  control<-list()
-  for(arg in names(formals(sys.function())))
-    control[arg]<-list(get(arg))
+                      parallel.version.check=TRUE,
+                      parallel.inherit.MT=FALSE){
 
+  control <- handle.controls("control.san")
   set.control.class("control.san")
 }
