@@ -1,12 +1,12 @@
-#  File R/ergm.san.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File R/ergm.san.R in package ergm, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2003-2020 Statnet Commons
-#######################################################################
+#  Copyright 2003-2021 Statnet Commons
+################################################################################
 
 #' Use Simulated Annealing to attempt to match a network to a vector of mean
 #' statistics
@@ -58,13 +58,14 @@
 #' @param object Either a [`formula`] or an [`ergm`] object. The
 #'   [`formula`] should be of the form \code{y ~ <model terms>}, where
 #'   \code{y} is a network object or a matrix that can be coerced to a
-#'   [`network`] object.  For the details on the
-#'   possible \code{<model terms>}, see \code{\link{ergm-terms}}.  To
-#'   create a \code{\link[network]{network}} object in , use the
+#'   [`network`] object.  For the details on the possible \code{<model
+#'   terms>}, see \code{\link{ergm-terms}}.  To create a
+#'   \code{\link[network]{network}} object in , use the
 #'   \code{network()} function, then add nodal attributes to it using
 #'   the \code{\%v\%} operator if necessary.
 #' @return A network or list of networks that hopefully have network
-#'   statistics close to the \code{target.stats} vector.
+#'   statistics close to the \code{target.stats} vector. Additionally,
+#'   [attr()]-style attributes `formula` and `stats` are included.
 #' @keywords models
 #' @aliases san.default
 #' @export
@@ -99,19 +100,18 @@ san.default <- function(object,...)
 #' chain.  If NULL, this is taken to be the network named in the formula.
 #'
 #' @param output Character, one of `"network"` (default),
-#'   `"edgelist"`, or `"pending_update_network"`: determines the
+#'   `"edgelist"`, or `"ergm_state"`: determines the
 #'   output format. Partial matching is performed.
 #' @param only.last if `TRUE`, only return the last network generated;
 #'   otherwise, return a [`network.list`] with `nsim` networks.
 #'
-#' @param control A list of control parameters for algorithm tuning; see
-#' \code{\link{control.san}}.
-#' @param verbose Logical or numeric giving the level of verbosity. Higher values produce more verbose output.
+#' @templateVar mycontrol control.san
+#' @template control
+#' @template verbose
+#'
 #' @param offset.coef A vector of offset coefficients; these must be passed in by the user.  
 #' Note that these should be the same set of coefficients one would pass to \code{ergm} via 
-#' its \code{offset.coef} argument.  For example, if one has a curved \code{offset(gwesp)} 
-#' term in the model, then the \code{decay} parameter should \emph{not} be passed in via 
-#' \code{offset.coef}.
+#' its \code{offset.coef} argument.
 #' @param \dots Further arguments passed to other functions.
 #' @examples
 #' \donttest{
@@ -164,14 +164,14 @@ san.default <- function(object,...)
 #' @export
 san.formula <- function(object, response=NULL, reference=~Bernoulli, constraints=~., target.stats=NULL,
                         nsim=NULL, basis=NULL,
-                        output=c("network","edgelist","pending_update_network"),
+                        output=c("network","edgelist","ergm_state"),
                         only.last=TRUE,
                         control=control.san(),
                         verbose=FALSE, 
                         offset.coef=NULL,
                         ...) {
   check.control.class("san", "san")
-  control.toplevel(...,myname="san")
+  handle.control.toplevel("san", ...)
 
   output <- match.arg(output)
 
@@ -191,45 +191,43 @@ san.formula <- function(object, response=NULL, reference=~Bernoulli, constraints
   }
 
   nw <- as.network(ensure_network(nw), populate=FALSE)
-  # nw is now a network/pending_update_network hybrid class. As long
+  # nw is now a network/ergm_state hybrid class. As long
   # as its edges are only accessed through methods that
-  # pending_update_network methods overload, it should be fine.
+  # ergm_state methods overload, it should be fine.
+  if(!is(nw,"ergm_state")) ergm_preprocess_response(nw, response)
+
+  # Inherit constraints from nw if needed.
+  tmp <- .handle.auto.constraints(nw, constraints, NULL, NULL)
+  nw <- tmp$nw; constraints <- tmp$constraints
+
+  proposal<-ergm_proposal(constraints,arguments=control$SAN.prop.args,nw=nw, hints=control$SAN.prop, weights=control$SAN.prop.weights, class="c",reference=reference, term.options=control$term.options)
+  model <- ergm_model(formula, nw, extra.aux=list(proposal=proposal$auxiliaries), term.options=control$term.options)
+  proposal$aux.slots <- model$slots.extra.aux$proposal
 
-  proposal<-ergm_proposal(constraints,arguments=control$SAN.prop.args,nw=nw,weights=control$SAN.prop.weights, class="c",reference=reference,response=response)
-  model <- ergm_model(formula, nw, response=response, term.options=control$term.options)
-    
-  ## need to extract post-reviseinit theta offsets here
-  model.initial <- ergm_model(formula, nw, response=response, initialfit=TRUE, term.options=control$term.options)  
   
-  if(length(offset.coef) != sum(model.initial$etamap$offsettheta)) {
-    stop("Length of ", sQuote("offset.coef"), " in SAN is ", length(offset.coef), ", while the number of offset coefficients in the model is ", sum(model.initial$etamap$offsettheta), ".")  
+  if(length(offset.coef) != sum(model$etamap$offsettheta)) {
+    stop("Length of ", sQuote("offset.coef"), " in SAN is ", length(offset.coef), ", while the number of offset coefficients in the model is ", sum(model$etamap$offsettheta), ".")  
   }
   
   if(any(is.na(offset.coef))) {
     stop("Missing offset coefficients passed to SAN.")
   }
   
-  init.coefs <- numeric(nparam(model.initial, canonical=FALSE))
-  names(init.coefs) <- param_names(model.initial)
-  init.coefs[model.initial$etamap$offsettheta] <- offset.coef
-  coefs <- ergm.reviseinit(model, init.coefs)
-  offset.coef <- coefs[model$etamap$offsettheta]
-    
-  san(model, response=response, reference=reference, constraints=proposal, target.stats=target.stats, nsim=nsim, basis=nw, output=output, only.last=only.last, control=control, verbose=verbose, offset.coef=offset.coef, ...)
+  san(model, reference=reference, constraints=proposal, target.stats=target.stats, nsim=nsim, basis=nw, output=output, only.last=only.last, control=control, verbose=verbose, offset.coef=offset.coef, ...)
 }
 
 #' @describeIn san A lower-level function that expects a pre-initialized [`ergm_model`].
 #' @export
-san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constraints=~., target.stats=NULL,
+san.ergm_model <- function(object, reference=~Bernoulli, constraints=~., target.stats=NULL,
                            nsim=NULL, basis=NULL,
-                           output=c("network","edgelist","pending_update_network"),
+                           output=c("network","edgelist","ergm_state"),
                            only.last=TRUE,
                            control=control.san(),
                            verbose=FALSE, 
                            offset.coef=NULL,
                            ...) {
   check.control.class("san", "san")
-  control.toplevel(...,myname="san")
+  handle.control.toplevel("san", ...)
 
   output <- match.arg(output)
   model <- object
@@ -242,7 +240,7 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
     if(nsim>1 && !is.null(control$seed)) warn("Setting the random seed with nsim>1 will produce a list of identical networks.")
     if(nsim>1){
       return(structure(replicate(nsim,
-                                 san(object, response=response, reference=reference, constraints=constraints, target.stats=target.stats,
+                                 san(object, reference=reference, constraints=constraints, target.stats=target.stats,
                                      basis=basis,
                                      output=output,
                                      only.last=only.last,
@@ -258,9 +256,9 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
   if(!is.null(control$seed)) set.seed(as.integer(control$seed))
   nw <- basis
   nw <- as.network(ensure_network(nw), populate=FALSE)
-  # nw is now a network/pending_update_network hybrid class. As long
+  # nw is now a network/ergm_state hybrid class. As long
   # as its edges are only accessed through methods that
-  # pending_update_network methods overload, it should be fine.
+  # ergm_state methods overload, it should be fine.
 
   if(is.null(target.stats)){
     stop("You need to specify target statistic via",
@@ -268,11 +266,18 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
   }
 
   proposal <- if(inherits(constraints, "ergm_proposal")) constraints
-              else ergm_proposal(constraints,arguments=control$SAN.prop.args,
-                                 nw=nw, weights=control$SAN.prop.weights, class="c",reference=reference,response=response)
+              else{
+                # Inherit constraints from nw if needed.
+                tmp <- .handle.auto.constraints(nw, constraints, NULL, NULL)
+                nw <- tmp$nw; constraints <- tmp$constraints
+                ergm_proposal(constraints,arguments=control$SAN.prop.args,
+                              nw=nw, hints=control$SAN.prop, weights=control$SAN.prop.weights, class="c",reference=reference, term.options=control$term.options)
+              }
 
-  ## we have the post-reviseinit offset thetas
-  ## need to remap them to etas using ergm.eta
+  if(length(proposal$auxiliaries) && !length(model$slots.extra.aux$proposal))
+    stop("The proposal appears to be requesting auxiliaries, but the initialized model does not export any proposal auxiliaries.")
+
+  ## need to remap thetas to etas using ergm.eta
   ## then just keep the offsets because we don't
   ## care about the non-offset coefs
   coefs <- numeric(nparam(model, canonical=FALSE))
@@ -288,19 +293,19 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
   if(control$SAN.ignore.finite.offsets) offsets[is.finite(offsets)] <- 0
   
   noffset <- sum(offset.indicators)
-    
-  Clist <- ergm.Cprepare(nw, model, response=response)
   
   if (verbose) {
-    message(paste("Starting ",control$SAN.maxit," MCMC iteration", ifelse(control$SAN.maxit>1,"s",""),
+    message(paste("Starting ",control$SAN.maxit," SAN iteration", ifelse(control$SAN.maxit>1,"s",""),
         " of ", control$SAN.nsteps,
         " steps", ifelse(control$SAN.maxit>1, " each", ""), ".", sep=""))
   }
-  maxedges <- max(control$SAN.init.maxedges, Clist$nedges)
-  netsumm <- summary(model,nw,response=response)[!offset.indicators]
+  netsumm<-summary(model,nw)[!offset.indicators]
   target.stats <- vector.namesmatch(target.stats, names(netsumm))
   stats <- netsumm-target.stats
-  control$invcov <- diag(1/(nparam(model, canonical=TRUE) - noffset), nparam(model, canonical=TRUE) - noffset)
+  invcov.dim <- nparam(model, canonical=TRUE) - noffset
+  NVL(control$SAN.invcov) <- diag(1/invcov.dim, invcov.dim)
+  if(!is.SPD(control$SAN.invcov) || nrow(control$SAN.invcov) != invcov.dim)
+    stop(sQuote("control$SAN.invcov"), " parameter should be a square matrix of dimension equal to the number of non-offset statistics")
 
   nstepss <-
     (if(is.function(control$SAN.nsteps.alloc)) control$SAN.nsteps.alloc(control$SAN.maxit) else control$SAN.nsteps.alloc) %>%
@@ -308,7 +313,8 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
     (function(x) x/sum(x) * control$SAN.nsteps) %>%
     round()
 
-  z <- NULL
+  state <- ergm_state(nw, model=model, proposal=proposal, stats=stats)
+  sm <- NULL
   for(i in 1:control$SAN.maxit){
     if (verbose) {
       message(paste("#", i, " of ", control$SAN.maxit, ": ", sep=""),appendLF=FALSE)
@@ -321,24 +327,23 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
     # even if control$SAN.ignore.finite.offsets is FALSE
     if(abs(tau) < .Machine$double.eps) offsets[is.finite(offsets)] <- 0
     
-    z <- ergm_SAN_slave(Clist, proposal, stats, tau, control, verbose,..., prev.run=z, nsteps=nsteps, statindices=statindices, offsetindices=offsetindices, offsets=offsets)
+    z <- ergm_SAN_slave(state, tau, control, verbose,..., nsteps=nsteps, statindices=statindices, offsetindices=offsetindices, offsets=offsets)
+    state <- z$state
+    sm <- rbind(sm, z$s)
+    sm.prop <- z$s.prop
 
     if(z$status!=0) stop("Error in SAN.")
     
-    outnw <- pending_update_network(nw,z,response=response)
-    nw <-  outnw
-    stats <- z$s[nrow(z$s),]
+    stats <- sm[nrow(sm),]
     # Use *proposal* distribution of statistics for weights.
     invcov <-
-      if(control$SAN.invcov.diag) ginv(diag(diag(cov(z$s.prop)), ncol(z$s.prop)), tol=.Machine$double.eps)
-      else ginv(cov(z$s.prop), tol=.Machine$double.eps)
+      if(control$SAN.invcov.diag) ginv(diag(diag(cov(sm.prop)), ncol(sm.prop)), tol=.Machine$double.eps)
+      else ginv(cov(sm.prop), tol=.Machine$double.eps)
 
-    # On resuming, don't accumulate proposal record.
-    z$s.prop <- NULL
     # Ensure no statistic has weight 0:
     diag(invcov)[abs(diag(invcov))<.Machine$double.eps] <- min(diag(invcov)[abs(diag(invcov))>=.Machine$double.eps],1)
     invcov <- invcov / sum(diag(invcov)) # Rescale for consistency.
-    control$invcov <- invcov
+    control$SAN.invcov <- invcov
     
     if(verbose){
       message("SAN summary statistics:")
@@ -348,18 +353,18 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
       message("Difference: SAN target.stats - Goal target.stats =")
       message_print(stats)
       message("New statistics scaling =")
-      message_print(diag(control$invcov))
+      message_print(diag(control$SAN.invcov))
       message("Scaled Mahalanobis distance = ", mahalanobis(stats, 0, invcov, inverted=TRUE))
     }
     
+    out.mat <- z$s
+    attr(out.mat, "W") <- invcov
     if(!only.last){
       out.list[[i]] <- switch(output,
-                              pending_update_network=outnw,
-                              network=as.network(outnw),
-                              edgelist=as.edgelist(outnw)
+                              ergm_state=state,
+                              network=as.network(state),
+                              edgelist=as.edgelist(state)
                               )
-      out.mat <- z$s
-      attr(out.mat, "W") <- invcov
     }else{
       if(i<control$SAN.maxit && isTRUE(all.equal(unname(stats), numeric(length(stats))))){
         if(verbose) message("Target statistics matched exactly.")
@@ -368,14 +373,17 @@ san.ergm_model <- function(object, response=NULL, reference=~Bernoulli, constrai
     }
   }
   if(control$SAN.maxit > 1 && !only.last){
-    structure(out.list, formula = formula, networks = out.list, 
+    structure(out.list, formula = formula,
               stats = out.mat, class="network.list")
   }else{
-    switch(output,
-           pending_update_network=outnw,
-           network=as.network(outnw),
-           edgelist=as.edgelist(outnw)
-           )    
+    structure(
+      switch(output,
+             ergm_state=state,
+             network=as.network(state),
+             edgelist=as.edgelist(state)
+             ),
+      stats = out.mat
+    )
   }
 }
 
@@ -385,9 +393,9 @@ san.ergm <- function(object, formula=object$formula,
                      constraints=object$constraints, 
                      target.stats=object$target.stats,
                      nsim=NULL, basis=NULL,
-                     output=c("network","edgelist","pending_update_network"),
+                     output=c("network","edgelist","ergm_state"),
                      only.last=TRUE,
-                     control=object$control$SAN.control,
+                     control=object$control$SAN,
                      verbose=FALSE, 
                      offset.coef=NULL,
                      ...) {
@@ -398,109 +406,72 @@ san.ergm <- function(object, formula=object$formula,
               basis=basis,
               reference = object$reference,
               output=output,
+              only.last=only.last,
               constraints=constraints,
               control=control,
               verbose=verbose, 
               offset.coef=offset.coef, ...)
 }
 
-ergm_SAN_slave <- function(Clist,proposal,stats,tau,control,verbose,...,prev.run=NULL, nsteps=NULL, samplesize=NULL, maxedges=NULL, statindices=NULL, offsetindices=NULL, offsets=NULL) {
-  if(is.null(prev.run)){ # Start from Clist
-    nedges <- c(Clist$nedges,0,0)
-    tails <- Clist$tails
-    heads <- Clist$heads
-    weights <- Clist$weights
-  }else{ # Pick up where we left off
-    nedges <- prev.run$newnwtails[1]
-    tails <- prev.run$newnwtails[2:(nedges+1)]
-    heads <- prev.run$newnwheads[2:(nedges+1)]
-    weights <- prev.run$newnwweights[2:(nedges+1)]
-    nedges <- c(nedges,0,0)
-    stats <- prev.run$s[nrow(prev.run$s),]
-  }
+#' Internal Function to Perform Simulated Annealing
+#'
+#' This is an internal function, not normally called directly by the
+#' user. The \code{ergm_SAN_slave} function samples networks and
+#' network statistics using a simulated annealing (SAN) algorithm via
+#' \code{SAN_wrapper}.
+#' 
+#' @param state an [`ergm_state`] representing the sampler state, containing information about the network, the model, the proposal, and current statistics.
+#'
+#' @templateVar mycontrol control.san
+#' @param tau a scalar; temperature to use; higher temperature means more proposals that "worsen" the statistics are accepted.
+#' @param nsteps an integer; number of SAN proposals.
+#' @param samplesize an integer; number of network statistics to return.
+#' @param statindices,offsetindices,offsets specification for offset handling; see [san.formula()] implementation.
+#' @template control
+#' @template verbose
+#' @param ... additional arguments, currently unused.
+#' @keywords internal
+#' @export
+ergm_SAN_slave <- function(state, tau,control,verbose, ..., nsteps=NULL, samplesize=NULL, statindices=NULL, offsetindices=NULL, offsets=NULL){
+  on.exit(ergm_Cstate_clear())
 
+  state$proposal$flags$SAN <- TRUE
   if(is.null(nsteps)) nsteps <- control$SAN.nsteps
   if(is.null(samplesize)) samplesize <- control$SAN.samplesize
-  if(is.null(maxedges)) maxedges <- control$SAN.init.maxedges
 
-  repeat{
-    if(is.null(Clist$weights)){
-      z <- .C("SAN_wrapper",
-              as.integer(nedges),
-              as.integer(tails), as.integer(heads),
-              as.integer(Clist$n),
-              as.integer(Clist$dir), as.integer(Clist$bipartite),
-              as.integer(Clist$nterms),
-              as.character(Clist$fnamestring),
-              as.character(Clist$snamestring),
-              as.character(proposal$name), as.character(proposal$pkgname),
-              as.double(c(Clist$inputs,proposal$inputs)), as.double(deInf(tau)),
-              s = as.double(c(stats, numeric(length(stats)*(samplesize-1)))), s.prop = double(length(stats)*samplesize),
-              as.integer(samplesize),
-              as.integer(nsteps), 
-              newnwtails = integer(maxedges),
-              newnwheads = integer(maxedges),
-              as.double(control$invcov),
-              as.integer(verbose), as.integer(proposal$arguments$constraints$bd$attribs),
-              as.integer(proposal$arguments$constraints$bd$maxout), as.integer(proposal$arguments$constraints$bd$maxin),
-              as.integer(proposal$arguments$constraints$bd$minout), as.integer(proposal$arguments$constraints$bd$minin),
-              as.integer(proposal$arguments$constraints$bd$condAllDegExact), as.integer(length(proposal$arguments$constraints$bd$attribs)),
-              as.integer(maxedges),
-              status = integer(1),
-              nstats=as.integer(length(statindices)),
-              statindices=as.integer(statindices - 1),
-              noffsets=as.integer(length(offsetindices)),
-              offsetindices=as.integer(offsetindices - 1),
-              offsets=as.double(deInf(offsets)),
-              PACKAGE="ergm")
-      
-        # save the results (note that if prev.run is NULL, c(NULL$s,z$s)==z$s.
-      z<-list(s=matrix(z$s, ncol=length(statindices), byrow = TRUE), s.prop=matrix(z$s.prop, ncol=length(statindices), byrow = TRUE),
-                newnwtails=z$newnwtails, newnwheads=z$newnwheads, status=z$status, maxedges=maxedges)
+  z <-
+    if(!is.valued(state)){
+      .Call("SAN_wrapper",
+            state,
+            # SAN settings
+            as.double(deInf(tau)),
+            as.integer(samplesize),
+            as.integer(nsteps),
+            as.double(control$SAN.invcov),
+            statindices=as.integer(statindices - 1),
+            offsetindices=as.integer(offsetindices - 1),
+            offsets=as.double(deInf(offsets)),
+            as.integer(verbose),
+            PACKAGE="ergm")
     }else{
-      z <- .C("WtSAN_wrapper",
-              as.integer(nedges),
-              as.integer(tails), as.integer(heads), as.double(weights),
-              as.integer(Clist$n),
-              as.integer(Clist$dir), as.integer(Clist$bipartite),
-              as.integer(Clist$nterms),
-              as.character(Clist$fnamestring),
-              as.character(Clist$snamestring),
-              as.character(proposal$name), as.character(proposal$pkgname),
-              as.double(c(Clist$inputs,proposal$inputs)), as.double(deInf(tau)),
-              s = as.double(c(stats, numeric(length(stats)*(samplesize-1)))), s.prop = double(length(stats)*samplesize),
-              as.integer(samplesize),
-              as.integer(nsteps), 
-              newnwtails = integer(maxedges),
-              newnwheads = integer(maxedges),
-              newnwweights = double(maxedges),
-              as.double(control$invcov),
-              as.integer(verbose), 
-              as.integer(maxedges),
-              status = integer(1),
-              nstats=as.integer(length(statindices)),
-              statindices=as.integer(statindices - 1),
-              noffsets=as.integer(length(offsetindices)),
-              offsetindices=as.integer(offsetindices - 1),
-              offsets=as.double(deInf(offsets)),              
-              PACKAGE="ergm")
-      # save the results
-      z<-list(s=matrix(z$s, ncol=length(statindices), byrow = TRUE), s.prop=matrix(z$s.prop, ncol=length(statindices), byrow = TRUE),
-              newnwtails=z$newnwtails, newnwheads=z$newnwheads, newnwweights=z$newnwweights, status=z$status, maxedges=maxedges)
+      .Call("WtSAN_wrapper",
+            state,
+            # SAN settings
+            as.double(deInf(tau)),
+            as.integer(samplesize),
+            as.integer(nsteps),
+            as.double(control$SAN.invcov),
+            statindices=as.integer(statindices - 1),
+            offsetindices=as.integer(offsetindices - 1),
+            offsets=as.double(deInf(offsets)),
+            as.integer(verbose),
+            PACKAGE="ergm")
     }
+  # save the results
+  z$s <- matrix(z$s, ncol=length(statindices), byrow = TRUE)
+  z$s.prop <- matrix(z$s.prop, ncol=length(statindices), byrow = TRUE)
+  colnames(z$s) <- colnames(z$s.prop) <- param_names(state, canonical=TRUE)[statindices]
+  z$state <- update(z$state)
 
-    
-    z$s <- rbind(prev.run$s,z$s)
-    z$s.prop <- rbind(prev.run$s.prop,z$s.prop)
-    
-    if(z$status!=1) return(z) # Handle everything except for MCMC_TOO_MANY_EDGES elsewhere.
-    
-    # The following is only executed (and the loop continued) if too many edges.
-    maxedges <- maxedges * 10
-    if(!is.null(control$SAN.max.maxedges)){
-      if(maxedges == control$SAN.max.maxedges*10) # True iff the previous maxedges exactly equaled control$SAN.max.maxedges and that was too small.
-        return(z) # This will kick the too many edges problem upstairs, so to speak.
-      maxedges <- min(maxedges, control$SAN.max.maxedges)
-    }
-  }
+  z
 }
