@@ -1,12 +1,12 @@
-#  File R/is.curved.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File R/is.curved.R in package ergm, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2003-2020 Statnet Commons
-#######################################################################
+#  Copyright 2003-2021 Statnet Commons
+################################################################################
 ###################################################################
 ## This file has utilities whose primary purpose is examining or ##
 ## manipulating ERGM formulas.                                   ##
@@ -38,7 +38,7 @@ is.curved.NULL <- function(object, ...) FALSE # By convention.
 #' @describeIn ergm_model Tests whether the model is curved.
 #' @export
 is.curved.ergm_model <- function(object, ...){
-  any(object$etamap$canonical==0)
+  length(object$etamap$curved)>0
 }
 
 #' @rdname is.curved 
@@ -53,13 +53,14 @@ is.curved.formula<-function(object,response=NULL,basis=NULL,...){
   }
   
   nw <- ensure_network(nw)
+  ergm_preprocess_response(nw, response)
   
-  m<-ergm_model(object, nw, response=response, ...)
+  m<-ergm_model(object, nw, ...)
   is.curved(m)
 }
 
 #' @rdname is.curved 
 #' @export
 is.curved.ergm<-function(object,...){
-  any(object$etamap$canonical==0)
+  length(object$etamap$curved)>0
 }
