@@ -1,22 +1,23 @@
-/*  File src/changestats_coincidence.c in package ergm, part of the Statnet suite
- *  of packages for network analysis, https://statnet.org .
+/*  File src/changestats_coincidence.c in package ergm, part of the
+ *  Statnet suite of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  https://statnet.org/attribution
+ *  https://statnet.org/attribution .
  *
- *  Copyright 2003-2020 Statnet Commons
+ *  Copyright 2003-2021 Statnet Commons
  */
-#include "changestats_coincidence.h"
+#include "ergm_edgetree.h"
+#include "ergm_changestat.h"
 
 /*****************
  changestat: d_coincidence
 *****************/
-D_CHANGESTAT_FN(d_coincidence) { 
+C_CHANGESTAT_FN(c_coincidence) { 
   /*The inputparams are assumed to be set up as follows:
     The first 2*nstats values are in pairs:  (degree, attrvalue)
     The values following the first 2*nstats values are the nodal attributes.*/
-  int i, echange;
+  int echange;
   Vertex b1, b2, index;
   Vertex film1, film2, act;
   Vertex nb, nb1, nb2;
@@ -26,9 +27,7 @@ D_CHANGESTAT_FN(d_coincidence) {
   nb  = 2*nb2-1;
 
   /* *** don't forget act -> film1 */    
-  ZERO_ALL_CHANGESTATS(i);
-  FOR_EACH_TOGGLE(i) {
-    echange = IS_OUTEDGE(act=TAIL(i), film1=HEAD(i)) ? -1 : 1;
+    echange = IS_OUTEDGE(act=tail, film1=head) ? -1 : 1;
     b1 = film1-nb1;
     STEP_THROUGH_OUTEDGES(act, e, film2) {
      if(film2 != film1){
@@ -41,7 +40,4 @@ D_CHANGESTAT_FN(d_coincidence) {
      if(INPUT_PARAM[index-1]>0.0) CHANGE_STAT[(int)(INPUT_PARAM[index-1])-1] += echange;
      }
     }
-    TOGGLE_IF_MORE_TO_COME(i); /* Needed in case of multiple toggles */
-  }
-  UNDO_PREVIOUS_TOGGLES(i); /* Needed on exit in case of multiple toggles */
 }
