@@ -1,13 +1,14 @@
-/*  File src/changestats_internal.c in package ergm, part of the Statnet suite
- *  of packages for network analysis, https://statnet.org .
+/*  File src/changestats_internal.c in package ergm, part of the
+ *  Statnet suite of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  https://statnet.org/attribution
+ *  https://statnet.org/attribution .
  *
- *  Copyright 2003-2020 Statnet Commons
+ *  Copyright 2003-2021 Statnet Commons
  */
-#include "changestats_internal.h"
+#include "ergm_edgetree.h"
+#include "ergm_changestat.h"
 
 /***************************************************************
  changestats internal
