@@ -1,11 +1,11 @@
-/*  File src/geodist.h in package ergm, part of the Statnet suite
- *  of packages for network analysis, https://statnet.org .
+/*  File src/geodist.h in package ergm, part of the
+ *  Statnet suite of packages for network analysis, https://statnet.org .
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) at
- *  https://statnet.org/attribution
+ *  https://statnet.org/attribution .
  *
- *  Copyright 2003-2020 Statnet Commons
+ *  Copyright 2003-2021 Statnet Commons
  */
 #ifndef GEODIST_H
 #define GEODIST_H
@@ -21,12 +21,4 @@ void full_geodesic_distribution (int *edgelist, int *nnodes,
 				 int *nodecolor, int *dist, int *Q,
 				 int *geodist);
 
-void geodesic_matrix (int *edgelist, int *nnodes,
-		      int *nodelist, int *nedges, 
-		      int *nodecolor, int *distmat, int *Q);
-
-void pair_geodesic (int *edgelist, int *nnodes, int *nodelist,
-                     int *nedges, int *nodecolor, int *dist, 
-		    int *Q, int *source, int *destination);
-
 #endif
