@@ -1,12 +1,12 @@
-#  File tests/drop_tests.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File tests/drop_tests.R in package ergm, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2003-2020 Statnet Commons
-#######################################################################
+#  Copyright 2003-2021 Statnet Commons
+################################################################################
 library(statnet.common)
 opttest({
 library(ergm)
@@ -58,6 +58,6 @@ stopifnot(all.equal(truth, coef(mined.mcmc), check.attributes=FALSE, tolerance=0
 set.seed(1)
 y <- network.initialize(10, directed=FALSE)
 y[1,2]<-y[2,3]<-y[3,4]<-1
-dummy <- ergm(y~edges+triangles+degree(2)+kstar(5)+gwdegree(1,fixed=FALSE),
-              control=control.ergm(MCMLE.maxit=3)) # It doesn't seem to stop for a while.
+dummy <- ergm(y~edges+triangles+degree(2)+kstar(2)+kstar(5)+gwdegree,
+              control=control.ergm(MCMLE.maxit=3), verbose=TRUE) # It doesn't seem to stop for a while.
 }, "drop")
