@@ -1,12 +1,12 @@
-#  File tests/termTests.directed.R in package ergm, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File tests/termTests.directed.R in package ergm, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2003-2020 Statnet Commons
-#######################################################################
+#  Copyright 2003-2021 Statnet Commons
+################################################################################
 
 library(ergm)
 
@@ -34,13 +34,13 @@ s.ak <- summary(samplike~asymmetric("group", levels=3))
 e.ak <- ergm(samplike~asymmetric(function(x) x %v% "group", levels=3), estimate="MPLE")
 s.adk <- summary(samplike~asymmetric(~group, diff=TRUE, keep=1:2))
 e.adk <- ergm(samplike~asymmetric("group", diff=TRUE, keep=c(1,3)), estimate="MPLE")
-if (s.0 != 32 || round(e.0$coef+1.33,3)!=0 ||
-    s.a != 17 || round(e.a$coef+.6008,3)!=0  ||
+if (s.0 != 32 || round(coef(e.0)+1.33,3)!=0 ||
+    s.a != 17 || round(coef(e.a)+.6008,3)!=0  ||
     !all(s.ad==c(7,2,8)) ||
-    !all(round(e.ad$coef+c(.6931, .6931, .4855),3)==0) ||
-    s.ak!=8 || round(e.ak$coef+.4855,3)!=0 ||
+    !all(round(coef(e.ad)+c(.6931, .6931, .4855),3)==0) ||
+    s.ak!=8 || round(coef(e.ak)+.4855,3)!=0 ||
     !all(s.adk==c(7,2)) ||
-    !all(round(e.adk$coef+c(.6931, .4855),3)==0)) {
+    !all(round(coef(e.adk)+c(.6931, .4855),3)==0)) {
  print(list(s.0=s.0, e.0=e.0, s.a=s.a, e.a=e.a, s.ad=s.ad, e.ad=e.ad, s.ak=s.ak,
             e.ak=e.ak, s.adk=s.adk, e.adk=e.adk))
  stop("Failed asymmetric term test")
@@ -60,10 +60,10 @@ s.a <- summary(samplike~ctriple(function(x) x %v% "group"))
 e.a <- ergm(samplike~ctriple("group"), estimate="MPLE")
 s.ad <- summary(samplike~ctriad(~group, diff=TRUE))
 e.ad <- ergm(samplike~ctriple(function(x) x %v% "group", diff=TRUE), estimate="MPLE")
-if (s.0 != 39 || round(e.0$coef + .3522, 3) != 0 ||
-    s.a != 34 || round(e.a$coef - .1217, 3) != 0 ||
+if (s.0 != 39 || round(coef(e.0) + .3522, 3) != 0 ||
+    s.a != 34 || round(coef(e.a) - .1217, 3) != 0 ||
     !all(s.ad==c(8,4,22)) ||
-    !all(round(e.ad$coef+c(.1949, -.6931, -.2023),3)==0)) {
+    !all(round(coef(e.ad)+c(.1949, -.6931, -.2023),3)==0)) {
  print(list(s.0=s.0, e.0=e.0, s.a=s.a, e.a=e.a, s.ad=s.ad, e.ad=e.ad))
  stop("Failed ctriple term test")
 } else {
@@ -79,9 +79,9 @@ s.0 <- summary(samplike~cyclicalties)
 e.0 <- ergm(samplike~cyclicalties, estimate="MPLE")
 s.b <- summary(samplike~cyclicalties(attr=function(x) x %v% "group"))
 e.b <- ergm(samplike~cyclicalties(attr="group"), estimate="MPLE")
-if (s.0 != 62 || round(e.0$coef + 0.4154, 3) != 0 ||
+if (s.0 != 62 || round(coef(e.0) + 0.4154, 3) != 0 ||
 		!all(s.b==55) ||
-		!all(round(e.b$coef - 0.2289 , 3) == 0)) {
+		!all(round(coef(e.b) - 0.2289 , 3) == 0)) {
 	print(list(s.0=s.0, e.0=e.0, s.b=s.b, e.b=e.b))
 	stop("Failed cyclicalties term test")
 } else {
@@ -97,7 +97,7 @@ s.0 <- summary(samplike~idegrange(5:8))
 e.0 <- ergm(samplike~idegrange(5:8), estimate="MPLE")
 s.h <- summary(samplike~idegrange(5:8, by="group", homophily=TRUE))
 e.h <- ergm(samplike~idegrange(5:8, by=~group, homophily=TRUE), estimate="MPLE")
-if (!all(s.0==c(9, 6, 4, 3)) || round(e.0$coef + c(-0.1431, 1.0986, 1.1451, 0.2231 ))!= 0 ||!all(s.h==c(5, 3, 0, 0)) || round(e.h$coef + c(-0.5108, -2.1972 , Inf, Inf ))!= 0) {
+if (!all(s.0==c(9, 6, 4, 3)) || round(coef(e.0) + c(-0.1431, 1.0986, 1.1451, 0.2231 ))!= 0 ||!all(s.h==c(5, 3, 0, 0)) || round(coef(e.h) + c(-0.5108, -2.1972 , Inf, Inf ))!= 0) {
 	print(list(s.0=s.0, e.0=e.0, s.h=s.h, e.h=e.h))
 	stop("Failed idegrange term test")
 } else {
@@ -111,18 +111,18 @@ if (!all(s.0==c(9, 6, 4, 3)) || round(e.0$coef + c(-0.1431, 1.0986, 1.1451, 0.22
 
 # gwidegree, directed
 num.tests=num.tests + 1
-s.d <- summary(samplike~gwidegree(.3))
+s.d <- summary(samplike~gwidegree())
 e.d <- ergm(samplike~gwidegree(.4, fixed=TRUE), estimate="MPLE")
 s.df <- summary(samplike~gwidegree(.3, fixed=TRUE))
 e.df <- ergm(samplike~gwidegree(.2, fixed=TRUE), estimate="MPLE")
 s.dfa <- summary(samplike~gwidegree(.1, TRUE, "group"))
 e.dfa <- ergm(samplike~gwidegree(.5, TRUE, function(x) x %v% "group"), estimate="MPLE")
 if (!all(head(s.d)==c(0,3,5,1,3,2)) ||
-    round(e.d$coef + 5.783202, 3) != 0 ||
+    round(coef(e.d) + 5.783202, 3) != 0 ||
     round(s.df - 23.89614, 3) != 0 ||
-    round(e.df$coef + 2.247936, 3) != 0 ||
+    round(coef(e.df) + 2.247936, 3) != 0 ||
     !all(round(s.dfa-c(7.715119, 4.408762, 7.734290),3)==0) ||
-    !all(round(e.dfa$coef+c(5.460448, 5.754111, 6.144961),3)==0)) {
+    !all(round(coef(e.dfa)+c(5.460448, 5.754111, 6.144961),3)==0)) {
  print(list(s.d=head(s.d), e.d=e.d, s.df=s.df, e.df=e.df, e.da=e.da, s.dfa=s.dfa, e.dfa=e.dfa))
  stop("Failed gwidegree term test")
 } else {
@@ -133,18 +133,18 @@ if (!all(head(s.d)==c(0,3,5,1,3,2)) ||
 
 # gwodegree, directed
 num.tests=num.tests + 1
-s.d <- summary(samplike~gwodegree(.3))
+s.d <- summary(samplike~gwodegree())
 e.d <- ergm(samplike~gwodegree(.4, fixed=TRUE), estimate="MPLE")
 s.df <- summary(samplike~gwodegree(.3, fixed=TRUE))
 e.df <- ergm(samplike~gwodegree(.2, fixed=TRUE), estimate="MPLE")
 s.dfa <- summary(samplike~gwodegree(.1, TRUE, ~group))
 e.dfa <- ergm(samplike~gwodegree(.5, TRUE, "group"), estimate="MPLE")
 if (!all(head(s.d)==c(0,0,1,5,7,5)) ||
-    round(e.d$coef + 1.990492, 3) != 0 ||
+    round(coef(e.d) + 1.990492, 3) != 0 ||
     round(s.df - 24.23040, 3) != 0 ||
-    round(e.df$coef - 43.61801, 3) != 0 ||
+    round(coef(e.df) - 43.61801, 3) != 0 ||
     !all(round(s.dfa-c(7.735906, 4.419631, 7.736070),3)==0) ||
-    !all(round(e.dfa$coef+c(4.1860720, 5.9706455, 0.4921623),3)==0)) {
+    !all(round(coef(e.dfa)+c(4.1860720, 5.9706455, 0.4921623),3)==0)) {
  print(list(s.d=s.d, e.d=e.d, s.df=s.df, e.df=e.df, e.da=e.da, s.dfa=s.dfa, e.dfa=e.dfa))
  stop("Failed gwodegree term test")
 } else {
@@ -153,34 +153,6 @@ if (!all(head(s.d)==c(0,0,1,5,7,5)) ||
 }
 
 
-
-
-# hammingmix, directed
-num.tests=num.tests + 1
-set.seed(32)
-nodes <- trunc(runif(65, 1, 18))
-nodes2 <- trunc(runif(65, 1,18))                
-el <- cbind(nodes, nodes2)
-el[46,1] <- 3
-# if x is not specified, summaries should be 0
-s.a <- summary(samplike~hammingmix("group"))
-s.ax <- summary(samplike~hammingmix(function(x) x %v% "group", x=el))
-e.ax <- ergm(samplike~hammingmix(~group, x=el), estimate="MPLE")
-s.axb <- summary(samplike~hammingmix(function(x) x %v% "group", el, levels2=-(2:6)))
-e.axb <- ergm(samplike~hammingmix("group", el, base=c(1,2,5,6,8,9)), estimate="MPLE")
-if (!all(s.a == 0) ||
-    !all(s.ax==c(36, 0, 8, 4, 18, 2, 16, 12, 50)) ||
-    !all(round(e.ax$coef[2:4]+c(1.0986, .2876, 2.5649),3)==0) ||
-    !all(s.axb==c(36,16,12,50)) ||
-    !all(round(e.axb$coef+c(.28768, 2.5649, .91629),3) ==0)) {
- print(list(s.a=s.a, s.ax=s.ax, e.ax=e.ax, s.axb=s.axb, e.axb=e.axb))
- stop("Failed hammingmix term test")
-} else {
-  num.passed.tests=num.passed.tests+1
-  print("Passed hammingmix term test")
-}
-
-
 # idegree, directed
 num.tests=num.tests + 1
 s.d <- summary(samplike~idegree(2:3))
@@ -189,10 +161,10 @@ s.db <- summary(samplike~idegree(1:3, "group"))
 e.db <- ergm(samplike~idegree(3, function(x) x %v% "group"), estimate="MPLE")
 s.dbh <- summary(samplike~idegree(4:5, "group", TRUE))
 e.dbh <- ergm(samplike~idegree(2, ~group, TRUE), estimate="MPLE")
-if (!all(s.d==c(3,5)) || round(e.d$coef - 1.223775, 3) != 0 ||
+if (!all(s.d==c(3,5)) || round(coef(e.d) - 1.223775, 3) != 0 ||
     !all(s.db==c(0,2,1,0,1,2,0,0,2)) ||
-    !all(round(e.db$coef-c(-0.6931472, 0.8183103, 17.4324836),3)==0) ||
-    !all(s.dbh==c(3,2)) || round(e.dbh$coef - .3448,3) !=0) {
+    !all(round(coef(e.db)-c(-0.6931472, 0.8183103, 17.4324836),3)==0) ||
+    !all(s.dbh==c(3,2)) || round(coef(e.dbh) - .3448,3) !=0) {
  print(list(s.d=s.d, e.d=e.d, s.db=s.db, e.db=e.db, s.dbh=s.dbh, e.dbh=e.dbh))
  stop("Failed idegree term test")
 } else {
@@ -205,7 +177,7 @@ if (!all(s.d==c(3,5)) || round(e.d$coef - 1.223775, 3) != 0 ||
 num.tests=num.tests + 1
 s.0 <- summary(samplike~intransitive)
 e.0 <- ergm(samplike~intransitive, estimate="MPLE")
-if (s.0 != 224 || round(e.0$coef + .21, 3) != 0) {
+if (s.0 != 224 || round(coef(e.0) + .21, 3) != 0) {
  print(list(s.0=s.0, e.0=e.0))
  stop("Failed intransitive term test")
 } else {
@@ -219,7 +191,7 @@ if (s.0 != 224 || round(e.0$coef + .21, 3) != 0) {
 num.tests=num.tests + 1
 s.0 <- summary(samplike~idegree1.5)
 e.0 <- ergm(samplike~idegree1.5, estimate="MPLE")
-if (round(s.0-214.6543,3) != 0 || round(e.0$coef + .2387, 3) != 0) {
+if (round(s.0-214.6543,3) != 0 || round(coef(e.0) + .2387, 3) != 0) {
  print(list(s.0=s.0, e.0=e.0))
  stop("Failed idegree1.5 term test")
 } else {
@@ -235,8 +207,8 @@ e.k <- ergm(samplike~istar(c(2,4)), estimate="MPLE")
 s.ka <- summary(samplike~istar(2, function(x) x %v% "group"))
 e.ka <- ergm(samplike~istar(2, "group"), estimate="MPLE")
 if (!all(s.k == c(88,233,455)) ||
-    round(e.k$coef - c(-.28615, .02477), 3) != 0 ||
-    s.ka != 100 || round(e.ka$coef - .2401, 3) != 0) {
+    round(coef(e.k) - c(-.28615, .02477), 3) != 0 ||
+    s.ka != 100 || round(coef(e.ka) - .2401, 3) != 0) {
  print(list(s.k=s.k, e.k=e.k, s.ka=s.ka, e.ka=e.ka))
  stop("Failed istar term test")
 } else {
@@ -249,7 +221,7 @@ if (!all(s.k == c(88,233,455)) ||
 num.tests=num.tests + 1
 s.0 <- summary(samplike~m2star)
 e.0 <- ergm(samplike~m2star, estimate="MPLE")
-if (s.0 != 378 || round(e.0$coef + .1028, 3) != 0) {
+if (s.0 != 378 || round(coef(e.0) + .1028, 3) != 0) {
  print(list(s.0=s.0, e.0=e.0))
  stop("Failed m2star term test")
 } else {
@@ -273,14 +245,14 @@ s.sk <- summary(samplike~mutual(same="group", levels=2))
 e.sk <- ergm(samplike~mutual(same=~group, levels=1), estimate="MPLE")
 s.bk <- summary(samplike~mutual(by="Trinity", keep=2))
 e.bk <- ergm(samplike~mutual(by="Trinity", keep=2:3), estimate="MPLE")
-if (s.0 != 28 || round(e.0$coef - .5596, 3) != 0 ||
-    s.s != 23 || round(e.s$coef - .9954, 3) != 0 ||
+if (s.0 != 28 || round(coef(e.0) - .5596, 3) != 0 ||
+    s.s != 23 || round(coef(e.s) - .9954, 3) != 0 ||
     !all(s.b==c(17,18,21)) ||
-    !all(round(e.b$coef-c(.0157, .0667, .8077),3)==0) ||
+    !all(round(coef(e.b)-c(.0157, .0667, .8077),3)==0) ||
     !all(s.sd==c(8,4,11)) ||
-    !all(round(e.sd$coef-c(.8267, 1.3863, 1.0116),3)==0) ||
-    s.sk != 4 || round(e.sk$coef - .8266, 3) != 0 ||
-    s.bk != 18 || !all(round(e.bk$coef - c(.0714, .8108)) == 0)){
+    !all(round(coef(e.sd)-c(.8267, 1.3863, 1.0116),3)==0) ||
+    s.sk != 4 || round(coef(e.sk) - .8266, 3) != 0 ||
+    s.bk != 18 || !all(round(coef(e.bk) - c(.0714, .8108)) == 0)){
  print(list(s.0=s.0, e.0=e.0,s.s=s.s, e.s=e.s, s.b=s.b, e.b=e.b,
             s.sd=s.sd, e.sd=e.sd, s.sk=s.sk, e.sk=e.sk,
             s.bk=s.bk, e.bk=e.bk))
@@ -295,7 +267,7 @@ if (s.0 != 28 || round(e.0$coef - .5596, 3) != 0 ||
 num.tests=num.tests + 1
 s.0 <- summary(samplike~nearsimmelian)
 e.0 <- ergm(samplike~nearsimmelian, estimate="MPLE")
-if (s.0 != 18 || round(e.0$coef + .4366483, 3) != 0) {
+if (s.0 != 18 || round(coef(e.0) + .4366483, 3) != 0) {
  print(list(s.0=s.0, e.0=e.0))
  stop("Failed nearsimmelian term test")
 } else {
@@ -311,8 +283,8 @@ e.a <- ergm(samplike~nodeicov(function(x) x %v% "YearsServed"), estimate="MPLE")
 s.at <- summary(samplike~nodeicov(~YearsServed^2))
 e.at <- ergm(samplike~nodeicov(~(.%v%"YearsServed")^2), estimate="MPLE")
 s.att <- summary(samplike~nodeicov(~poly(YearsServed,2,raw=TRUE)))
-if (s.a != 439 || round(e.a$coef + .1739, 3) != 0 ||
-    s.at != 2345 || round(e.at$coef + .02805, 3) != 0 ||
+if (s.a != 439 || round(coef(e.a) + .1739, 3) != 0 ||
+    s.at != 2345 || round(coef(e.at) + .02805, 3) != 0 ||
     any(s.att != c(439,2345))) {
  print(list(s.a=s.a, e.a=e.a, s.at=s.at, e.at=e.at, s.att=s.att))
  stop("Failed nodeicov term test")
@@ -329,9 +301,9 @@ e.a <- ergm(samplike~nodeifactor(~group), estimate="MPLE")
 s.ab <- summary(samplike~nodeifactor(function(x) x %v% "Trinity", levels=TRUE))
 e.ab <- ergm(samplike~nodeifactor("Trinity", base=(2:3)), estimate="MPLE")
 if (!all(s.a==c(13,46)) ||
-    !all(round(e.a$coef+c(1.4424, .4618),3)==0) ||
+    !all(round(coef(e.a)+c(1.4424, .4618),3)==0) ||
     !all(s.ab==c(28, 29, 31)) ||
-    round(e.ab$coef+ .9719, 3) != 0) {
+    round(coef(e.ab)+ .9719, 3) != 0) {
   print(list(s.a=s.a,e.a=e.a, s.ab=s.ab, e.ab=e.ab))
   stop("Failed nodeifactor term test")
 } else {
@@ -348,8 +320,8 @@ e.a <- ergm(samplike~nodeocov(function(x) x %v% "YearsServed"), estimate="MPLE")
 s.at <- summary(samplike~nodeocov(~YearsServed^2))
 e.at <- ergm(samplike~nodeocov(~(.%v%"YearsServed")^2), estimate="MPLE")
 s.att <- summary(samplike~nodeocov(~poly(YearsServed,2,raw=TRUE)))
-if (s.a != 467 || round(e.a$coef + .1581, 3) != 0 ||
-    s.at != 2691 || round(e.at$coef + .02243, 3) != 0 ||
+if (s.a != 467 || round(coef(e.a) + .1581, 3) != 0 ||
+    s.at != 2691 || round(coef(e.at) + .02243, 3) != 0 ||
     any(s.att != c(467,2691))) {
  print(list(s.a=s.a, e.a=e.a, s.at=s.at, e.at=e.at, s.att=s.att))
  stop("Failed nodeocov term test")
@@ -366,9 +338,9 @@ e.a <- ergm(samplike~nodeofactor(~group), estimate="MPLE")
 s.ab <- summary(samplike~nodeofactor("Trinity", levels=TRUE))
 e.ab <- ergm(samplike~nodeofactor(function(x) x %v% "Trinity", base=(2:3)), estimate="MPLE")
 if (!all(s.a==c(18,36)) ||
-    !all(round(e.a$coef+c(1.0217, .8353),3)==0) ||
+    !all(round(coef(e.a)+c(1.0217, .8353),3)==0) ||
     !all(s.ab==c(31,30,27)) ||
-    round(e.ab$coef+ .8287, 3) != 0) {
+    round(coef(e.ab)+ .8287, 3) != 0) {
   print(list(s.a=s.a,e.a=e.a, s.ab=s.ab, e.ab=e.ab))
   stop("Failed nodeofactor term test")
 } else {
@@ -385,10 +357,10 @@ s.db <- summary(samplike~odegree(1:3, function(x) x %v% "group"))
 e.db <- ergm(samplike~odegree(4, "group"), estimate="MPLE")
 s.dbh <- summary(samplike~odegree(4:5, ~group, TRUE))
 e.dbh <- ergm(samplike~odegree(2, "group", TRUE), estimate="MPLE")
-if (!all(s.d==c(0,1)) || round(e.d$coef + .1625189, 3) != 0 ||
+if (!all(s.d==c(0,1)) || round(coef(e.d) + .1625189, 3) != 0 ||
     !all(s.db==c(0,0,0,0,0,1,0,0,0)) ||
-    !all(round(e.db$coef+c(-1.6292, 0.1112, 0.1625),3)==0) ||
-    !all(s.dbh==c(6,3)) || round(e.dbh$coef + 1.344,3) !=0) {
+    !all(round(coef(e.db)+c(-1.6292, 0.1112, 0.1625),3)==0) ||
+    !all(s.dbh==c(6,3)) || round(coef(e.dbh) + 1.344,3) !=0) {
  print(list(s.d=s.d, e.d=e.d, s.db=s.db, e.db=e.db,
             s.dbh=s.dbh, e.dbh=e.dbh))
  stop("Failed odegree term test")
@@ -406,8 +378,8 @@ e.k <- ergm(samplike~ostar(c(2,4)), estimate="MPLE")
 s.ka <- summary(samplike~ostar(2, "group"))
 e.ka <- ergm(samplike~ostar(2, function(x) x %v% "group"), estimate="MPLE")
 if (!all(s.k == c(88,178, 191)) ||
-    round(e.k$coef - c(.1224, -.1986), 3) != 0 ||
-    s.ka != 88 || round(e.ka$coef - .1466, 3) != 0) {
+    round(coef(e.k) - c(.1224, -.1986), 3) != 0 ||
+    s.ka != 88 || round(coef(e.ka) - .1466, 3) != 0) {
  print(list(s.k=s.k, e.k=e.k, s.ka=s.ka, e.ka=e.ka))
  stop("Failed ostar term test")
 } else {
@@ -422,7 +394,7 @@ if (!all(s.k == c(88,178, 191)) ||
 num.tests=num.tests + 1
 s.0 <- summary(samplike~odegree1.5)
 e.0 <- ergm(samplike~odegree1.5, estimate="MPLE")
-if (round(s.0-196.9432,3) != 0 || round(e.0$coef + 0.2909, 3) != 0) {
+if (round(s.0-196.9432,3) != 0 || round(coef(e.0) + 0.2909, 3) != 0) {
  print(list(s.0=s.0, e.0=e.0))
  stop("Failed odegree1.5 term test")
 } else {
@@ -437,7 +409,7 @@ s.0 <- summary(samplike~odegrange(5:8))
 e.0 <- ergm(samplike~odegrange(5:8), estimate="MPLE")
 s.h <- summary(samplike~odegrange(5:8, by=function(x) x %v% "group", homophily=TRUE))
 e.h <- ergm(samplike~odegrange(5:8, by="group", homophily=TRUE), estimate="MPLE")
-if (!all(s.0==c(12, 5, 0, 0)) || round(e.0$coef + c(0.619, -1.030, Inf, Inf))!= 0 ||!all(s.h==c(3, 0, 0, 0)) || round(e.h$coef + c(-0.2231, Inf , Inf, Inf ))!= 0) {
+if (!all(s.0==c(12, 5, 0, 0)) || round(coef(e.0) + c(0.619, -1.030, Inf, Inf))!= 0 ||!all(s.h==c(3, 0, 0, 0)) || round(coef(e.h) + c(-0.2231, Inf , Inf, Inf ))!= 0) {
 	print(list(s.0=s.0, e.0=e.0, s.h=s.h, e.h=e.h))
 	stop("Failed odegrange term test")
 } else {
@@ -454,11 +426,11 @@ e.0 <- ergm(samplike~receiver, estimate="MPLE")
 s.b <- summary(samplike~receiver(nodes=-(2:16)))
 e.b <- ergm(samplike~receiver(nodes=-(3:18)), estimate="MPLE")
 if (!all(s.0==c(8, 4, 2, 5, 3, 5, 7, 11, 10, 6, 3, 6, 3, 5, 3, 2, 3)) ||
-    !all(round(e.0$coef-c(-0.1178,-1.1787,-2.0149,-0.8755,-1.5404,-0.8755,
+    !all(round(coef(e.0)-c(-0.1178,-1.1787,-2.0149,-0.8755,-1.5404,-0.8755,
                           -0.3567, 0.6061, 0.3567,-0.6061,-1.5404,-0.6061,
                           -1.5404,-0.8755,-1.5404,-2.0149,-1.5404),3) == 0) ||
     !all(s.b==c(2,2,3)) ||
-    !all(round(e.b$coef- c(-2.0149, -0.1178),3)==0)) {
+    !all(round(coef(e.b)- c(-2.0149, -0.1178),3)==0)) {
  print(list(s.0=s.0, e.0=e.0, s.b=s.b, e.b=e.b))
  stop("Failed receiver term test")
 } else {
@@ -475,11 +447,11 @@ e.0 <- ergm(samplike~sender, estimate="MPLE")
 s.b <- summary(samplike~sender(nodes=-(2:16)))
 e.b <- ergm(samplike~sender(nodes=-(3:18)), estimate="MPLE")
 if (!all(s.0==c(5, 4, 4, 4, 5, 6, 4, 6, 5, 5, 6, 5, 5, 3, 5, 4, 6)) ||
-    !all(round(e.0$coef+c(0.8755,1.1787,1.1787,1.1787,0.8755,0.6061,1.1787,
+    !all(round(coef(e.0)+c(0.8755,1.1787,1.1787,1.1787,0.8755,0.6061,1.1787,
                           0.6061,0.8755,0.8755,0.6061,0.8755,0.8755,1.5404,
                           0.8755,1.1787,0.6061), 3) == 0) ||
     !all(s.b==c(6,4,6)) ||
-    !all(round(e.b$coef+c(.6061, .8755),3)==0)) {
+    !all(round(coef(e.b)+c(.6061, .8755),3)==0)) {
  print(list(s.0=s.0, e.0=e.0, s.b=s.b, e.b=e.b))
  stop("Failed sender term test")
 } else {
@@ -494,7 +466,7 @@ if (!all(s.0==c(5, 4, 4, 4, 5, 6, 4, 6, 5, 5, 6, 5, 5, 3, 5, 4, 6)) ||
 num.tests=num.tests + 1
 s.0 <- summary(samplike~simmelian)
 e.0 <- ergm(samplike~simmelian, estimate="MPLE")
-if (s.0 != 8 || round(e.0$coef - .6069, 3) != 0) {
+if (s.0 != 8 || round(coef(e.0) - .6069, 3) != 0) {
   print(list(s.0=s.0, e.0=e.0))
   stop("Failed simmelian term test")
 } else {
@@ -507,7 +479,7 @@ if (s.0 != 8 || round(e.0$coef - .6069, 3) != 0) {
 num.tests=num.tests + 1
 s.0 <- summary(samplike~simmelianties)
 e.0 <- ergm(samplike~simmelianties, estimate="MPLE")
-if (s.0 != 32 || round(e.0$coef - .1984, 3) != 0) {
+if (s.0 != 32 || round(coef(e.0) - .1984, 3) != 0) {
   print(list(s.0=s.0, e.0=e.0))
   stop("Failed simmelianties term test")
 } else {
@@ -521,7 +493,7 @@ if (s.0 != 32 || round(e.0$coef - .1984, 3) != 0) {
 num.tests=num.tests + 1
 s.0 <- summary(samplike~transitive)
 e.0 <- ergm(samplike~transitive, estimate="MPLE")
-if (s.0 != 154 || round(e.0$coef + .07745, 3) != 0) {
+if (s.0 != 154 || round(coef(e.0) + .07745, 3) != 0) {
  print(list(s.0=s.0, e.0=e.0))
  stop("Failed transitive term test")
 } else {
@@ -535,7 +507,7 @@ if (s.0 != 154 || round(e.0$coef + .07745, 3) != 0) {
 num.tests=num.tests + 1
 s.0 <- summary(samplike~transitiveties)
 e.0 <- ergm(samplike~transitiveties, estimate="MPLE")
-if (s.0 != 69 || round(e.0$coef + 0.4116, 3) != 0) {
+if (s.0 != 69 || round(coef(e.0) + 0.4116, 3) != 0) {
 	print(list(s.0=s.0, e.0=e.0))
 	stop("Failed transitiveties term test")
 } else {
@@ -553,10 +525,10 @@ s.a <- summary(samplike~ttriple("group"))
 e.a <- ergm(samplike~ttriple(~group), estimate="MPLE")
 s.ad <- summary(samplike~ttriad("group", diff=TRUE))
 e.ad <- ergm(samplike~ttriple(function(x) x %v% "group", diff=TRUE), estimate="MPLE")
-if (s.0 != 154 || round(e.0$coef + .07745, 3) != 0 ||
-    s.a != 121 || round(e.a$coef - .09518, 3) != 0 ||
+if (s.0 != 154 || round(coef(e.0) + .07745, 3) != 0 ||
+    s.a != 121 || round(coef(e.a) - .09518, 3) != 0 ||
     !all(s.ad==c(26,14,81)) ||
-    !all(round(e.ad$coef-c(-.05078, .38935, .13469),3)==0)) {
+    !all(round(coef(e.ad)-c(-.05078, .38935, .13469),3)==0)) {
  print(list(s.0=s.0, e.0=e.0, s.a=s.a, e.a=e.a, s.ad=s.ad, e.ad=e.ad))
  stop("Failed ttriple term test")
 } else {
